UNIT ClassicCalcul;

INTERFACE

uses
  SysUtils, ClassicDecla;

function DoubleHex(DD: double): string;
        { convertit un nombre rel (double)           }
        { en une chane de 16 caractres hexadcimaux }
function HexDouble(st: string): double;
        { convertit une chane de 16 caractres hexadcimaux }
        { en un nombre rel (double)                         }
function IntegerHex(nombre: Longint): string;
        { renvoie une chane de 8 caractres hexadcimaux }
function HexInteger(st: string): LongInt;
        { entre: chane de maximum 8 caractres hexadcimaux }
function CarHex(st: string): string;
        { convertit chaque caractre ASCII }
        { en deux caractres hexadcimaux  }
function HexCar(st: string): string;
        { convertit chaque paire de caractres hexadcimaux }
        { en un caractre ASCII                             }
function CodeOrdre(StCode: string): byte;
function IsoWord(CodeIso: string): word;
function IsoHex(IsoD: string): string;
         { entre: chane de chiffres dcimaux       }
         { sortie: chane de 8 chiffres hexadcimaux }
function IsoDec(IsoH: string): string;
         { entre: chane de chiffres hexadcimaux }
         { sortie: chane de 3 chiffres dcimaux   }
function TelH(st: string): string;
         { supprime les caractres non dcimaux, convertit en un nombre  }
         { hexadcimal les 14 chiffres les moins significatifs et ajoute }
         { des 0 devant le nombre pour obtenir 12 chiffres hexadcimaux  }
function dateH: string;
         { renvoie jjmaa avec le jour (jj) et l'anne (aa) en dcimal }
         {               et le mois (m) en hexadcimal                }

IMPLEMENTATION

uses
  ClassicMath;

FUNCTION DoubleHex(DD: double): string;
        { convertit un nombre rel (double)           }
        { en une chane de 16 caractres hexadcimaux }
type
  ArByte=array[0..7] of byte;
var
  bt,V,i: integer;
  ch0,ch1: char;
begin
  result:='';
  for i:=0 to 7 do begin
    bt:=ArByte(DD)[i];
    V:= bt mod 16;
    if V>9 then ch0:=chr(V+$37) else ch0:=chr(V+$30);
    V:=bt div 16;
    if V>9 then ch1:=chr(V+$37) else ch1:=chr(V+$30);
    result:=result+ch1+ch0 end;
end;

FUNCTION HexDouble(st: string): double;
        { convertit une chane de 16 caractres hexadcimaux }
        { en un nombre rel (double)                         }
type
  ArByte=array[1..8] of byte;
var
  VArByte: ArByte;
  ch: char;
  V0,V1,i: integer;
begin
  for i:=1 to (length(st) div 2) do begin
    ch:=st[i*2-1];
    case ch of
      '0'..'9': V1:=ord(ch)-$30;
      'A'..'F': V1:=ord(ch)-$37
      else V1:=0;
    end;
    ch:=st[i*2];
    case ch of
      '0'..'9': V0:=ord(ch)-$30;
      'A'..'F': V0:=ord(ch)-$37
      else V0:=0;
    end;
    VArByte[i]:=(V1*16+V0) end;
  result:=double(VArByte);
end;

FUNCTION IntegerHex(nombre: Longint): string;
        { renvoie une chane de 8 caractres hexadcimaux }
var
  nw: ar_w;
  ArNombre: array[1..2] of word absolute nombre;
begin
  nw[1]:=ArNombre[1];
  nw[2]:=ArNombre[2];
  result:=BinHex(nw,2);
end;

FUNCTION HexInteger(st: string): LongInt;
        { entre: chane de maximum 8 caractres hexadcimaux }
var
  nw: ar_w;
  lg:integer;
  ArResult: array[1..2] of word absolute result;
begin
  entreeH(st,nw,lg);
  ArResult[1]:=nw[1];
  if lg>1 then ArResult[2]:=nw[2]
  else ArResult[2]:=0;
end;

FUNCTION CarHex(st: string): string;
        { convertit chaque caractre ASCII }
        { en deux caractres hexadcimaux  }
var
  Oc,V,i: byte;
  ch0,ch1: char;
begin
  result:='';
  for i:=1 to length(st) do begin
    Oc:=ord(st[i]);
    V:= Oc mod 16;
    if V>9 then ch0:=chr(V+$37) else ch0:=chr(V+$30);
    V:=Oc div 16;
    if V>9 then ch1:=chr(V+$37) else ch1:=chr(V+$30);
    result:=result+ch1+ch0 end;
end;

FUNCTION HexCar(st: string): string;
        { convertit chaque paire de caractres hexadcimaux }
        { en un caractre ASCII                             }
var
  ch: char;
  V0,V1,i: byte;
begin
  result:='';
  for i:=1 to (length(st) div 2) do begin
    ch:=st[i*2-1];
    case ch of
      '0'..'9': V1:=ord(ch)-$30;
      'A'..'F': V1:=ord(ch)-$37
      else V1:=0;
    end;
    ch:=st[i*2];
    case ch of
      '0'..'9': V0:=ord(ch)-$30;
      'A'..'F': V0:=ord(ch)-$37
      else V0:=0;
    end;
    result:=result+chr(V1*16+V0) end;
end;

FUNCTION CodeOrdre(StCode: string): byte;
var
  val1,val2:integer;
begin
  val1:=ord(StCode[1]);
  val2:=ord(StCode[2]);
  if val1>$40 then dec(val1,7);
  if val2>$40 then dec(val2,7);
  result:=(val1-$30)*16+val2-$30;
end;

FUNCTION IsoWord(CodeIso: string): word;
var
  ErreurVal: integer;
begin
  val(CodeIso,result,ErreurVal);
end;

FUNCTION IsoHex(IsoD: string): string;
         { entre: chane de chiffres dcimaux       }
         { sortie: chane de 8 chiffres hexadcimaux }
begin
  result:=integerHex(IsoWord(IsoD));
end;

FUNCTION IsoDec(IsoH: string): string;
         { entre: chane de chiffres hexadcimaux }
         { sortie: chane de 3 chiffres dcimaux   }
var
  val1,val2,val3:integer;
begin
  IsoH:=copy(IsoH,length(IsoH)-2,3);
  val1:=ord(IsoH[1]);
  val2:=ord(IsoH[2]);
  val3:=ord(IsoH[3]);
  if val1>$40 then dec(val1,7);
  if val2>$40 then dec(val2,7);
  if val3>$40 then dec(val3,7);
  str((val1-$30)*256+(val2-$30)*16+val3-$30,result);
  while length(result)<3 do result:='0'+result;
end;

FUNCTION TelH(st: string): string;
         { supprime les caractres non dcimaux, convertit en un nombre  }
         { hexadcimal les 14 chiffres les moins significatifs et ajoute }
         { des 0 devant le nombre pour obtenir 12 chiffres hexadcimaux  }
var
  nw: ar_w;
  lg: integer;
  i: integer;
begin
  For i:=length(st) DownTo 1 do begin
    if not (st[i] in ['0'..'9'])
    then st:=copy(st,1,i-1)+copy(st,i+1,length(st)-i) end;
  if length(st)>14 then st:=copy(st,length(st)-13,14);
  entreeD(st,nw,lg);
  st:=BinHex(nw,lg);
  while length(st)<12 do st:='0'+st;
  result:=copy(st,length(st)-11,12);
end;

FUNCTION dateH: string;
         { renvoie jjmaa avec le jour (jj) et l'anne (aa) en dcimal }
         {               et le mois (m) en hexadcimal                }
var
  aujourdhui: double;
  moisH: char;
  V,C: integer;
begin
  aujourdhui:=date;
  val(FormatDateTime('mm',aujourdhui),V,C);
  if V>9 then MoisH:=chr(V+$37)
  else MoisH:=chr(V+$30);
  dateH:=FormatDateTime('dd',aujourdhui)+MoisH+FormatDateTime('yy',aujourdhui);
end;

END.
