UNIT ClassicDlm8Demo;

INTERFACE

uses
ClassicDecla;

procedure sed8(var nVe,nCle,nVs: ar_w; num: boolean);
	{ application de l'algorithme SED (128 bits)		}
	{ entres: nVe (vecteur), nCle (clef non singulire)    }
	{ 	   num:=true pour SED63/30, false pour SED30/63	}
	{ sortie: nVs					        }
procedure sed8X(var nVe,nCle,nVdlm,nVs: ar_w; num: boolean);
	{ application de l'algorithme SED (128 bits)		}
	{ entres: nVe (vecteur), nCle (clef non singulire)    }
	{        : nVdlm (vecteur XOR intermdiaire)	        }
	{ 	   num:=true pour SED63/30, false pour SED30/63	}
	{ sortie: nVs					        }

Var
  VMsg8,VMsg7,VCle8,VCle7,VXor,VDlm8,VDlm7,VCpl8,VCpl7,VMul8,Vmul7: string;
  VXor8,VXor7,VSed8,VSed7: string;
  chiffre: boolean;

IMPLEMENTATION

uses
  ClassicMath8, ClassicMath7, ClassicCalcul8, ClassicDlm7;

procedure dlm8(var nVe,nCle,nVs: ar_w; num: boolean); forward;
	{ application de l'algorithme DLM8		        }
	{ entres: nVe (vecteur), nCle (clef non singulire)    }
	{ 	   num:=true pour l'algorithme 63 (lment 64), }
        {               false pour l'algorithme 30 (lment 97) }
	{ sortie: nVs				                }
procedure mam8(var nVe,nCle,nVs: ar_w); forward;

{------------------------------------------------------------------------------}

PROCEDURE sed8(var nVe,nCle,nVs: ar_w; num: boolean);
	{ application de l'algorithme SED (128 bits)		}
	{ entres: nVe (vecteur), nCle (clef non singulire)    }
	{ 	   num:=true pour SED63/30, false pour SED30/63	}
	{ sortie: nVs					        }
var
  nVi,nVii: ar_w;
begin
  dlm8(nVe,nCle,nVi,num);
  if num then begin
    if not parite(nVi) then complement8(nVi);
    mam8(nVi,nCle,nVii) end
  else begin
    mam8(nVi,nCle,nVii);
    if not parite(nVii) then complement8(nVii) end;
  dlm8(nVii,nCle,nVs,not num);
  voiler(nVi);
end;


PROCEDURE sed8X(var nVe,nCle,nVdlm,nVs: ar_w; num: boolean);
	{ application de l'algorithme SED (128 bits)	        }
	{ entres: nVe (vecteur), nCle (clef non singulire)    }
	{        : nVdlm (vecteur XOR intermdiaire)	        }
	{ 	   num:=true pour SED63/30, false pour SED30/63 }
	{ sortie: nVs					        }
var
  nVi,nVii: ar_w;
  i: integer;
begin
  chiffre:=num;
  dlm8(nVe,nCle,nVi,num);
  if num then begin
    if not parite(nVi) then complement8(nVi);
    VCpl8:=BinHex8(nVi);
    mam8(nVi,nCle,nVii);
    VMul8:=BinHex8(nVii);
    for i:=1 to 8 do nVii[i]:=nVii[i] xor nVdlm[i];
    VXor8:=BinHex8(nVii) end
  else begin
    for i:=1 to 8 do nVi[i]:=nVi[i] xor nVdlm[i];
    VXor8:=BinHex8(nVi);
    mam8(nVi,nCle,nVii);
    VMul8:=BinHex8(nVii);
    if not parite(nVii) then complement8(nVii);
    VCpl8:=BinHex8(nVii) end;
  dlm8(nVii,nCle,nVs,not num);
  voiler(nVi);
end;

procedure dlm8(var nVe,nCle,nVs: ar_w; num: boolean);
	{ application de l'algorithme DLM8		        }
	{ entres: nVe (vecteur), nCle (clef non singulire)    }
	{ 	   num:=true pour l'algorithme 63 (lment 64), }
        {               false pour l'algorithme 30 (lment 97) }
	{ sortie: nVs				                }
var
  nV,nW,nCl: ar_w;
  pVe,pCle,direct:boolean;
begin
  if conforme8(nVe) then begin
    vecteur8_7(nVe,nV,pVe);
    vecteur8_7(nCle,nCl,pCle);
    dlm(nV,nCl,nW,num);
    vecteur7_8(pVe,nW,nVs) end
  else move(nVe,nVs,16);
  if chiffre then direct:=num else direct:= not num;
  if direct then begin
    VMsg7:=BinHex8(nV);
    VCle7:=BinHex8(nCl);
    VDlm7:=BinHex8(nW);
    VDlm8:=BinHex8(nVs) end
  else begin
    if chiffre then VXor7:=BinHex8(nV) else VCpl7:=BinHex8(nV);
    VSed7:=BinHex8(nW);
    VSed8:=BinHex8(nVs) end;
  voiler(nV);
  voiler(nW);
  voiler(nCl);
end;

procedure mam8(var nVe,nCle,nVs: ar_w);
var
  nV,nW,nCl: ar_2w;
  nVV: ar_w absolute nV;
  nWW: ar_w absolute nW;
  nCll: ar_w absolute nCl;
  pVe,pCle:boolean;
begin
  if conforme8(nVe) then begin
    vecteur8_7(nVe,nVV,pVe);
    vecteur8_7(nCle,nCll,pCle);
    if chiffre then VCpl7:=BinHex8(nVV) else VXor7:=BinHex8(nVV);
    mam7(nV,nCl,nW);
    VMul7:=BinHex8(nWW);
    vecteur7_8(pVe,nWW,nVs) end
  else move(nVe,nVs,16);
  voiler(nVV);
  voiler(nWW);
  voiler(nCll);
end;

END.

